/**
 * SimpleGraph.java
 * LO42 - TP5 - Recherche de chemin de poids minimal et utilisation de jgraph
 * Martin ROLLAND - Laurent COUVIDOU
 */

/**
 * Importation des bibliothques
 */

import org.jgraph.JGraph;
import org.jgraph.graph.*;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.border.Border;
import javax.swing.undo.UndoableEdit;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;
import java.awt.Rectangle;
import java.awt.Color;
import java.util.*;

public class SimpleGraph extends JFrame {

	/**
	 * Variables globales
	 */
	// Nombre de sommets (attention !  tenir  jour...)
	static int nbSommets = 6;

	// Tableau des sommets
	static DefaultGraphCell[] sommet = new DefaultGraphCell[nbSommets];

	// Tableau des artes
	static DefaultEdge[][] arc = new DefaultEdge[nbSommets][nbSommets];

	// Poids de chaque arte (distance)
	static int[][] poids = new int[nbSommets][nbSommets];

	/**
	 * Dclaration du graphe, du modle et de la map
	 */
	JGraph myGraph;

	DefaultMonGraphModel myModel; // On utilise le modle du TP4

	Map myAttribute;

	/**
	 * Cration du graphe, du modle et de la map
	 */
	SimpleGraph() {
		myModel = new DefaultMonGraphModel();
		myGraph = new JGraph(myModel);
		myAttribute = new Hashtable();
	}

	/**
	 * Cration d'un sommet (cellule rectangulaire)
	 */ 
	DefaultGraphCell setVertix(String name, Rectangle bound, Color bgColor,
			boolean isOpaque, Border border, Color borderColor) {
		DefaultGraphCell myCell = new DefaultGraphCell(name);
		//Map cellAttribute = GraphConstants.createMap();
		Map cellAttribute = new AttributeMap();
		myAttribute.put(myCell, cellAttribute);
		//cellAttribute.put("valeur",new Integer(2));
		GraphConstants.setBounds(cellAttribute, bound);
		Object[] cells = new Object[] { myCell };
		myModel.insert(cells, myAttribute, null, null, null);

		if (bgColor != null)
			GraphConstants.setBackground(cellAttribute, bgColor);
		if (isOpaque)
			GraphConstants.setOpaque(cellAttribute, isOpaque);
		if (border != null)
			GraphConstants.setBorder(cellAttribute, border);
		if (borderColor != null)
			GraphConstants.setBorderColor(cellAttribute, borderColor);
		return myCell;
	}

	/**
	 * Cration d'une arte (entre 2 sommets)
	 */ 
	DefaultEdge setEdge(String name, int lineEnd, boolean isEndFill) {
		DefaultEdge myEdge = new DefaultEdge(name);
		//Map edgeAttribute = GraphConstants. createMap();
		Map edgeAttribute = new AttributeMap();
		myAttribute.put(myEdge, edgeAttribute);
		edgeAttribute.put("value", new Integer(4));
		//GraphConstants.setValue(edgeAttribute, new Integer(4));
		GraphConstants.setLineEnd(edgeAttribute, lineEnd);
		GraphConstants.setEndFill(edgeAttribute, isEndFill);
		GraphConstants.setLineColor(edgeAttribute, Color.blue);
		return myEdge;
	}

	/**
	 * Cration d'une arte (entre deux cellules) avec valeur et couleur en
	 * paramtres
	 */ 
	DefaultEdge setEdge(String name, int lineEnd, boolean isEndFill,
			int valeur, Color couleur) {
		DefaultEdge myEdge = new DefaultEdge(name);
		//Map edgeAttribute = GraphConstants. createMap();
		Map edgeAttribute = new AttributeMap();
		myAttribute.put(myEdge, edgeAttribute);
		Integer val = new Integer(valeur);
		edgeAttribute.put("value", val);
		//GraphConstants.setValue(edgeAttribute, new Integer(4));
		GraphConstants.setLineEnd(edgeAttribute, lineEnd);
		GraphConstants.setEndFill(edgeAttribute, isEndFill);
		GraphConstants.setLineColor(edgeAttribute, couleur);
		return myEdge;
	}

	/**
	 * Connexions
	 */
	void insertModel(Edge edge, Port source, Port target, ParentMap pm,
			UndoableEdit[] e) {
		ConnectionSet cs = new ConnectionSet(edge, source, target);
		Object[] cells = new Object[] { edge };
		myModel.insert(cells, myAttribute, cs, pm, e);
	}

	/**
	 * Affichage du graphe
	 */
	void showFrame() {
		this.getContentPane().add(new JScrollPane(myGraph));
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.pack();
		this.setVisible(true);
	}

	/**
	 * Cration simplifie d'un noeud
	 * @param nom
	 * @param x
	 * @param y
	 * @return
	 */
	DefaultGraphCell creerNoeud(String nom, int x, int y) {
		//System.out.println("Creation d'un noeud appel " + nom + "... ");
		DefaultGraphCell cell = setVertix(nom, new Rectangle(x, y, 60, 25),
				null, false, null, Color.black);
		//System.out.println("OK");
		return cell;
	}

	/**
	 * Cration simplifie d'une arte
	 * @param port1
	 * @param port2
	 * @param s1
	 * @param s2
	 * @param valeur
	 * @param couleur
	 * @return
	 */
	DefaultEdge creerArete(DefaultPort port1, DefaultPort port2, int s1,
			int s2, int valeur, Color couleur) {
		DefaultEdge edge = setEdge("arc" + s1 + "+" + s2,
				GraphConstants.ARROW_NONE, true, valeur, couleur);
		insertModel(edge, port1, port2, null, null);
		poids[s1][s2] = valeur;
		poids[s2][s1] = valeur;
		arc[s1][s2] = edge;
		arc[s2][s1] = edge;
		return edge;
	}

	/**
	 * Passage en bleu d'une arte
	 * Nota : BUG -> cliquer plusieurs fois pour faire apparatre la couleur
	 * @param edge
	 */
	void marquerArete(DefaultEdge edge) {
		Map edgeAttribute = new AttributeMap();
		myAttribute.put(edge, edgeAttribute);
		GraphConstants.setLineColor(edgeAttribute, Color.blue);
		edge.changeAttributes(edgeAttribute);
	}

	/**
	 * Entier -> Sommet
	 * @param s
	 * @return
	 */
	DefaultGraphCell donnerSommet(int s) {
		return sommet[s];
	}

	/**
	 * // Sommet -> Entier
	 * @param cell
	 * @return
	 */
	int donnerEntier(DefaultGraphCell cell) {
		int som = -1;
		for (int i = 0; i < nbSommets; i++) {
			if (sommet[i] == cell)
				som = i;
		}
		return som;
	}

	/**
	 * Plus court chemin entre deux sommets
	 * On utilise l'algo. de Dijskra (cf. schma), puis on slectionne le sommet
	 * d'arrive voulu parmi ceux trouvs  l'arrive
	 * Nota : sur le schma, en continu : les plus courts sous-chemins
	 * @param s1 s2
	 * @return chemin
	 */
	public List plusCourtChemin(int s1, int s2) {

		final int infini = Integer.MAX_VALUE;				// Valeur initiale des tiquettes
		int sommetEnCours = s1;								// Dernier sommet integr au nuage
		boolean[] sommetVisite = new boolean[nbSommets];	// Tableau indiquant pour chaque sommet s'il a
															// t visit ou non
		int[] etiquette = new int[nbSommets];				// tiquette ("d(u)") de chaque sommet
		int[] predecesseur = new int[nbSommets];			// Prdesseur de chaque sommet sur le plus court
															// sous-chemin
		
		int sommet;											// Variable de traitement
		int i;												// Idem
		boolean continuer = true;							// Idem
		
		// Initialisations
		for (i = 0; i < nbSommets; i++)	etiquette[i] = infini;
		etiquette[s1] = 0;
		for (i = 0; i < nbSommets; i++)	sommetVisite[i] = false;

		// Tant que tout n'a pas t visit
		while (continuer) {

			List liste = myModel.listeSucc(donnerSommet(sommetEnCours));
			Iterator it = liste.iterator();
			
			// Pour chaque successeur non visit du sommet en cours de traitement
			// -> mise  jour des tiquettes et du prdecesseur
			while (it.hasNext()) {
				i = Integer.parseInt(it.next().toString());
				System.out.println("Successeur du sommet "
						+ sommetEnCours + " en cours : " + i);
				// On limine les sommets visits
				if (!sommetVisite[i]) {
					// Si le chemin en cours est plus court on met  jour l'tiquette
					if (etiquette[i] > etiquette[sommetEnCours]
							+ poids[i][sommetEnCours]) {
						etiquette[i] = etiquette[sommetEnCours]
								+ poids[i][sommetEnCours];
						System.out.println("   Nouvelle etiquette : "
								+ etiquette[i]);
						// Successeurs des successeurs
						// -> mise  jour des prdecesseurs
						List liste2 = myModel.listeSucc(donnerSommet(i));
						Iterator it2 = liste2.iterator();
						int j;
						while (it2.hasNext()) {
							j = Integer.parseInt(it2.next().toString());
							// Le prdecesseur du successeur est le sommet en cours
							if (j == sommetEnCours)
								predecesseur[i] = j;
						}
					}
					else System.out.println("   Rien  changer : le chemin est plus long");
				}
				else System.out.println("   Rien  changer : sommet dj visit");
			}

			// Le sommet en cours est visit
			sommetVisite[sommetEnCours] = true;

			// On cherche le sommet de plus petite tiquette, c'est le sommet suivant  traiter
			int minEtiquette = infini;
			sommetEnCours = -1;
			for (i = 0; i < nbSommets; i++) {
				if ((etiquette[i] < minEtiquette) && (sommetVisite[i] == false)) {
					minEtiquette = etiquette[i];
					sommetEnCours = i;
				}
			}
			if (sommetEnCours == -1)
				continuer = false;
		}

		// Affichage pour info
		for (i = 0; i < nbSommets; i++)
			System.out.println("Etiquette de " + i + " : " + etiquette[i]);

		// On reconstitue le chemin  retourner (s1->s2)
		LinkedList chemin = new LinkedList();
		sommet = s2;
		chemin.add(donnerSommet(sommet));
		while (sommet != s1) {
			sommet = predecesseur[sommet];
			chemin.addFirst(donnerSommet(sommet));
		}
		return chemin;
	}

	/**
	 * Programme principal
	 * @param arg
	 */
	public static void main(String[] arg) {

		// *** Cration du graphe
		SimpleGraph graph = new SimpleGraph();

		// *** Cration des sommets
		// Cration de 0
		sommet[0] = graph.creerNoeud("0", 200, 20);
		DefaultPort port0 = new DefaultPort();
		sommet[0].add(port0);
		// Cration de 1
		sommet[1] = graph.creerNoeud("1", 40, 120);
		DefaultPort port1 = new DefaultPort();
		sommet[1].add(port1);
		// Cration de 2
		sommet[2] = graph.creerNoeud("2", 200, 120);
		DefaultPort port2 = new DefaultPort();
		sommet[2].add(port2);
		// Cration de 3
		sommet[3] = graph.creerNoeud("3", 360, 120);
		DefaultPort port3 = new DefaultPort();
		sommet[3].add(port3);
		// Cration de 4
		sommet[4] = graph.creerNoeud("4", 120, 220);
		DefaultPort port4 = new DefaultPort();
		sommet[4].add(port4);
		// Cration de 5
		sommet[5] = graph.creerNoeud("5", 280, 220);
		DefaultPort port5 = new DefaultPort();
		sommet[5].add(port5);

		// *** Cration des artes
		graph.creerArete(port0, port1, 0, 1, 8, Color.black);
		graph.creerArete(port0, port2, 0, 2, 2, Color.black);
		graph.creerArete(port0, port3, 0, 3, 4, Color.black);
		graph.creerArete(port1, port2, 1, 2, 7, Color.black);
		graph.creerArete(port2, port3, 2, 3, 1, Color.black);
		graph.creerArete(port1, port4, 1, 4, 2, Color.black);
		graph.creerArete(port2, port4, 2, 4, 3, Color.black);
		graph.creerArete(port2, port5, 2, 5, 9, Color.black);
		graph.creerArete(port3, port5, 3, 5, 5, Color.black);

		// Calcul du plus court chemin
		List chemin = graph.plusCourtChemin(0, 5);
		
		// Affichage en console pour info
		System.out.println("Chemin = " + chemin);

		// Marquage du plus court chemin
		// Nota : BUG -> cliquer plusieurs fois pour faire apparatre la couleur
		Iterator it = chemin.iterator();
		DefaultGraphCell sommet = (DefaultGraphCell) it.next();
		while (it.hasNext()) {
			graph.marquerArete(arc[graph.donnerEntier(sommet)][graph
					.donnerEntier(sommet = (DefaultGraphCell) it.next())]);
		}

		// Affichage du graphe
		graph.showFrame();
	}

} // Fin de SimpleGraph
